from django.http import HttpResponse, Http404
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.pagination import LimitOffsetPagination, PageNumberPagination
from .serializers import SciamImageSerializer
from django.shortcuts import render
from .models import SciamImage
from .enums import SUBJECTS, YEARS, TOPICS, TOPICS_STR, SEARCH_TYPES, SEARCH_TYPES_STR


def index(request):
    return render(request, 'sciamapp/index.html')

def test(request):
    return render(request, 'sciamapp/test_base.html')

def help(request):
    return render(request, 'sciamapp/help.html')

def about(request):
    return render(request, 'sciamapp/about.html')

def history(request):
    return render(request, 'sciamapp/history.html')

def under_construction(request):
    return render(request, 'sciamapp/under_construction.html')

def browse(request):
    # context = {'subjects':SUBJECTS, 'years':YEARS,}
    all_image_data = SciamImage.objects.all()
    if request.method == 'POST':
        selected_year = request.POST['year']
        selected_subject = request.POST['subject']

        if selected_year != 'null':
            all_imgs = SciamImage.objects.filter(date__year=selected_year)
        if selected_subject != 'null':
            if 'all_imgs' in locals():
                all_imgs = all_imgs.filter(subject=selected_subject)
            else:
                all_imgs = SciamImage.objects.filter(subject=selected_subject)
    if 'all_imgs' in locals():
       context = {'subjects': SUBJECTS, 'years': YEARS, 'images':all_imgs}

    else:
        context = {'subjects': SUBJECTS, 'years': YEARS, 'all_image_data': all_image_data,} 
    
    return render(request, 'sciamapp/browse.html', context)

def detail(request, image_id):
    try:
        image = SciamImage.objects.get(pk=image_id)
    except SciamImage.DoesNotExist:
        raise Http404("The requested Scientific American Illustration does not exist.")
    return render(request, 'sciamapp/detail.html', {'image': image,})

def detail_table(request, image_id_1):
    try:
        image = SciamImage.objects.get(pk=image_id_1)
    except SciamImage.DoesNotExist:
        raise Http404("The requested Scientific American Illustration does not exist.")
    return render(request, 'sciamapp/detail_table.html', {'image': image,})

def search(request):
    if request.method == 'POST':
        search_string = request.POST['search_string']
        search_type = '__contains'
        results_dict = []
        results = []

        for topic in TOPICS:
            custom_filter = topic + search_type
            results_dict.append(SciamImage.objects.filter(**{custom_filter:search_string}))

        for rt in results_dict:
            if rt is not None and rt is not []:
                results = results + list(set(list(rt)) - set(results))
        
        if results == []:
            context = {'search_string': search_string, 'no_results':'true', 'search_page':'true'}
        else:
            #results.sort(key=lambda x:x.id, reverse=False)
            context = {'search_string': request.POST['search_string'], 'images':results, 'search_page':'true'}

    else:
        context = {'topics': TOPICS_STR, 'types':SEARCH_TYPES_STR, 'search_page':'true'}

    return render(request, 'sciamapp/search.html', context)        


def search1(request):
    if request.method == 'POST':
        # if the user has searched something

        # save the search results 
        # by writing request.POST['<name attr of html tag in form>']
        # if request.POST['search_topic'] is not None and request.POST['search_type'] is not None and request.POST['search_string'] is not None:
            if request.POST['search_topic'] != "null":
                selected_topic = TOPICS[TOPICS_STR.index(request.POST['search_topic'])]
                search_type = SEARCH_TYPES[SEARCH_TYPES_STR.index(request.POST['search_type'])]
                search_string = request.POST['search_string']
                if selected_topic == 'year' or selected_topic == 'id':
                    search_type = '__contains'
                filter = selected_topic + search_type
                all_imgs = SciamImage.objects.filter(**{filter:search_string})
                context = {'topics': TOPICS_STR, 'types':SEARCH_TYPES_STR, 'previous':request.POST, 'images':all_imgs}
            else:
                context = {'topics': TOPICS_STR, 'types':SEARCH_TYPES_STR, 'error_st': 'no_search_topic'}
        # else:
        #         context = {'topics': TOPICS_STR, 'types':SEARCH_TYPES_STR, 'error_u': 'unknown'}
    else:
        context = {'topics': TOPICS_STR, 'types':SEARCH_TYPES_STR}

    return render(request, 'sciamapp/search1.html', context)

def search2(request):
    pass